@@ -35,6 +35,8 @@
 #include <libosso-abook/osso-abook-account-manager.h>
 #include <libosso-abook/osso-abook-settings.h>
 
+#include <rtcom-eventlogger-plugins/chat.h>
+
 enum
 {
     PROP_0,
@@ -99,6 +101,12 @@ struct _RTComLogModelPrivate
     RTComElQueryGroupBy group_by;
     gint limit;
 
+    /* whether to special-case groupchats (hide or
+     * fake normal message, depending on whether we know
+     * or not the contact sending the message, and depending
+     * on the protocol) */
+    gboolean show_group_chat;
+
     OssoABookWaitableClosure *aggregator_ready_closure;
     OssoABookAggregator * abook_aggregator;
     OssoABookContactSubscriptions * abook_subs;
@@ -419,6 +427,10 @@ new_discover_abook_contact (RTComLogModel *model, const char *local_uid,
     g_return_val_if_fail (osso_abook_waitable_is_ready(
           OSSO_ABOOK_WAITABLE(priv->abook_aggregator), NULL), NULL);
 
+    /* don't even attempt discovering hidden/empty numbers/IDs */
+    if ((remote_uid == NULL) || (*remote_uid == '\0'))
+        return NULL;
+
     vcard_field = vcard_field_for_account (model, local_uid);
     if (!vcard_field)
         return NULL;
@@ -442,12 +454,23 @@ new_discover_abook_contact (RTComLogModel *model, const char *local_uid,
 
     if (contacts)
     {
-        remote_ebook_uid = g_strdup (
-            osso_abook_contact_get_persistent_uid (OSSO_ABOOK_CONTACT (contacts->data)));
-        g_list_free (contacts);
+        /* only match if there is exactly one matching contact */
+        if (contacts->next == NULL)
+          {
+            g_debug ("%s: discovered remote contact: %s for remote uid %s on account %s",
+                G_STRFUNC, remote_ebook_uid, remote_uid, local_uid);
 
-        g_debug ("%s: discovered remote contact: %s for remote uid %s on account %s",
-            G_STRFUNC, remote_ebook_uid, remote_uid, local_uid);
+            remote_ebook_uid = g_strdup (
+                osso_abook_contact_get_persistent_uid (
+                    OSSO_ABOOK_CONTACT (contacts->data)));
+          }
+        else
+          {
+            g_debug ("%s: discovered multiple matching contacts, ignoring all",
+                G_STRFUNC);
+          }
+
+        g_list_free (contacts);
     }
 
     return remote_ebook_uid;
@@ -734,7 +757,7 @@ _stage_cached (gpointer data)
 
     struct _staging_data
     {
-        GValueArray * values;
+        GHashTable * values;
         GtkTreePath * path;
         GString * caching_account_data_key;
         account_data_t * account_data;
@@ -752,6 +775,8 @@ _stage_cached (gpointer data)
         gint count;
         const gchar * group_title;
         const gchar * event_type;
+        gboolean outgoing;
+        gint flags;
     } staging_data;
 
     GtkTreeIter iter, deletion_iter;
@@ -780,32 +805,67 @@ _stage_cached (gpointer data)
         {
             g_list_foreach(
                     caching_data->cached_events,
-                    (GFunc) g_value_array_free,
+                    (GFunc) g_hash_table_destroy,
                     NULL);
             g_list_free(caching_data->cached_events);
             g_free(caching_data);
             return FALSE;
         }
 
-        staging_data.values = (GValueArray *) events_iter->data;
+        staging_data.values = (GHashTable *) events_iter->data;
 
-        staging_data.event_id         = g_value_get_int(g_value_array_get_nth(staging_data.values, 0));
-        staging_data.service          = g_value_get_string(g_value_array_get_nth(staging_data.values, 1));
-        staging_data.group_uid        = g_value_get_string(g_value_array_get_nth(staging_data.values, 2));
-        staging_data.local_uid        = g_value_get_string(g_value_array_get_nth(staging_data.values, 3));
-        staging_data.remote_uid       = g_value_get_string(g_value_array_get_nth(staging_data.values, 4));
-        staging_data.remote_name      = g_value_get_string(g_value_array_get_nth(staging_data.values, 5));
-        staging_data.remote_ebook_uid = g_value_get_string(g_value_array_get_nth(staging_data.values, 6));
-        staging_data.text             = g_value_get_string(g_value_array_get_nth(staging_data.values, 7));
-        staging_data.icon_name        = g_value_get_string(g_value_array_get_nth(staging_data.values, 8));
-        staging_data.timestamp        = g_value_get_int(g_value_array_get_nth(staging_data.values, 9));
-        staging_data.count            = g_value_get_int(g_value_array_get_nth(staging_data.values, 10));
-        staging_data.group_title      = g_value_get_string(g_value_array_get_nth(staging_data.values, 11));
-        staging_data.event_type       = g_value_get_string(g_value_array_get_nth(staging_data.values, 12));
+#define LOOKUP_INT(x) \
+    g_value_get_int(g_hash_table_lookup(staging_data.values, x))
+#define LOOKUP_BOOL(x) \
+    g_value_get_boolean(g_hash_table_lookup(staging_data.values, x))
+#define LOOKUP_STR(x) \
+    g_value_get_string(g_hash_table_lookup(staging_data.values, x))
+
+        staging_data.event_id = LOOKUP_INT ("id");
+        staging_data.service = LOOKUP_STR ("service");
+        staging_data.group_uid = LOOKUP_STR ("group-uid");
+        staging_data.local_uid = LOOKUP_STR ("local-uid");
+        staging_data.remote_uid = LOOKUP_STR ("remote-uid");
+        staging_data.remote_name = LOOKUP_STR ("remote-name");
+        staging_data.remote_ebook_uid = LOOKUP_STR ("remote-ebook-uid");
+        staging_data.text = LOOKUP_STR ("content");
+        staging_data.icon_name = LOOKUP_STR ("icon-name");
+        staging_data.timestamp = LOOKUP_INT ("start-time");
+        staging_data.count = LOOKUP_INT ("event-count");
+        staging_data.group_title = LOOKUP_STR ("group-title");
+        staging_data.event_type = LOOKUP_STR ("event-type");
+        staging_data.outgoing = LOOKUP_BOOL ("outgoing");
+        staging_data.flags = LOOKUP_INT ("flags");
+
+        /* if requested, ignore groupchat events */
+        if (!priv->show_group_chat &&
+            (staging_data.flags & RTCOM_EL_FLAG_CHAT_GROUP) &&
+            !strcmp (staging_data.service, "RTCOM_EL_SERVICE_CHAT"))
+        {
+            /* this is ugly; some protocols use normal remote uids of contacts
+             * even for groupchat messages (notably, skype); if we ignore
+             * those and grouping is by uids or contact, we won't show *any*
+             * messages (incl. non-groupchat-ones) from that contact, because
+             * the database gave us only this row; in this
+             * case it's prefferable that we treat the groupchat message as
+             * a normal one; so, if abook euid of the contact is known, we'll
+             * treat the message as a normal one instead. otherwise, we ignore
+             * it. */
+            if (staging_data.remote_ebook_uid)
+              {
+                staging_data.flags &= ~RTCOM_EL_FLAG_CHAT_GROUP;
+              }
+            else
+              {
+                g_hash_table_destroy(staging_data.values);
+                continue;
+              }
+        }
 
         g_debug("Staging event:\n\tid: %d\n\tservice: %s\n\tgroup_uid: %s\n\tlocal_uid: %s\n\tremote_uid: %s\n\t"
                 "remote_name: %s\n\tremote_ebook_uid: %s\n\ttext: %s\n\ticon_name: %s\n\t"
-                "timestamp: %d\n\tevents in group: %d\n\tgroup title: %s\n\tevent type: %s\n",
+                "timestamp: %d\n\tevents in group: %d\n\tgroup title: %s\n\tevent type: %s\n\t"
+                "outgoing: %s\n\t flags: %d\n",
                 staging_data.event_id,
                 staging_data.service,
                 staging_data.group_uid,
@@ -818,7 +878,9 @@ _stage_cached (gpointer data)
                 staging_data.timestamp,
                 staging_data.count,
                 staging_data.group_title,
-                staging_data.event_type);
+                staging_data.event_type,
+                staging_data.outgoing ? "yes" : "no",
+                staging_data.flags);
 
         if(staging_data.icon_name)
         {
@@ -862,6 +924,8 @@ _stage_cached (gpointer data)
                 RTCOM_LOG_VIEW_COL_COUNT,          staging_data.count,
                 RTCOM_LOG_VIEW_COL_GROUP_TITLE,    staging_data.group_title,
                 RTCOM_LOG_VIEW_COL_EVENT_TYPE,     staging_data.event_type,
+                RTCOM_LOG_VIEW_COL_OUTGOING,       staging_data.outgoing,
+                RTCOM_LOG_VIEW_COL_FLAGS,          staging_data.flags,
 
                 /* We shouldn't be setting NULL for a GObject and we don't
                  * want to use separate set (to minimise row redraw), so we're
@@ -887,8 +951,8 @@ _stage_cached (gpointer data)
 
                 /* We want to put the discovered uid back to the staging data
                  * values so it gets freed afterwards, with the rest of them. */
-                g_value_take_string (g_value_array_get_nth(staging_data.values,
-                    6), (gchar *) staging_data.remote_ebook_uid);
+                g_value_take_string (g_hash_table_lookup (staging_data.values,
+                    "remote-ebook-uid"), (gchar *) staging_data.remote_ebook_uid);
             }
 
             c = _get_contact_from_abook_uid (model,
@@ -1090,7 +1154,7 @@ _stage_cached (gpointer data)
             }
         }
 
-        g_value_array_free(staging_data.values);
+        g_hash_table_destroy(staging_data.values);
     }
 
     g_list_free(caching_data->cached_events);
@@ -1105,7 +1169,7 @@ _threaded_cached_load (gpointer data)
     RTComLogModel * model = RTCOM_LOG_MODEL(data);
     RTComLogModelPrivate * priv = NULL;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
+    GHashTable * values = NULL;
 
     g_return_val_if_fail(RTCOM_IS_LOG_MODEL(model), FALSE);
 
@@ -1167,7 +1231,7 @@ _threaded_cached_load (gpointer data)
                     return NULL;
                 }
 
-                values = rtcom_el_iter_get_valuearray(
+                values = rtcom_el_iter_get_value_map(
                         it,
                         "id",
                         "service",
@@ -1182,6 +1246,8 @@ _threaded_cached_load (gpointer data)
                         "event-count",
                         "group-title",
                         "event-type",
+                        "outgoing",
+                        "flags",
                         NULL);
                 d->cached_events = g_list_prepend(d->cached_events, values);
             } while(rtcom_el_iter_next(it));
@@ -1203,59 +1269,6 @@ _threaded_cached_load (gpointer data)
     return NULL;
 }
 
-static void
-aggregator_contacts_changed_cb (OssoABookAggregator *aggregator,
-    OssoABookContact **contacts, RTComLogModel *model)
-{
-    /* Idea is, after the model has been populated, any added/removed
-     * contacts should be inspected to see whether we have some
-     * events that belong to those contacts, and update contact
-     * info, avatar, presence, etc. for those events. This however
-     * requires half of populate_pixbufs functionality, so until
-     * that's refactored, we just refresh the entire model. */
-
-    /* We're only interested in updates after the model was populated. */
-    if (!osso_abook_waitable_is_ready(OSSO_ABOOK_WAITABLE(aggregator), NULL))
-        return;
-
-    g_debug ("%s: contacts changed, refreshing RTComLogModel",
-        G_STRFUNC);
-
-    rtcom_log_model_refresh(model);
-#if 0
-    GtkTreeIter iter;
-    gboolean valid;
-
-
-    valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
-    while (valid)
-    {
-        gchar *remote_uid, *local_uid;
-        OssoABookContact *c;
-
-        gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
-            RTCOM_LOG_VIEW_COL_LOCAL_ACCOUNT, &local_uid,
-            RTCOM_LOG_VIEW_COL_REMOTE_ACCOUNT, &remote_uid,
-            RTCOM_LOG_VIEW_COL_CONTACT, &c, -1);
-
-        if (c == NULL)
-        {
-            gchar *remote_abook_uid = discover_abook_contact(model, local_uid,
-                remote_uid);
-
-            /* TODO: do something clever...possibly refactor _populate_pixbufs
-             * some more. */
-
-            g_free (remote_abook_uid);
-        }
-
-        g_free (remote_uid);
-        g_free (local_uid);
-
-        valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
-    }
-#endif
-}
 
 static void
 _connect_aggregator_signals (RTComLogModel *model)
@@ -1265,11 +1278,6 @@ _connect_aggregator_signals (RTComLogModel *model)
     priv->aggregator_ready_closure =
         osso_abook_waitable_call_when_ready (OSSO_ABOOK_WAITABLE(priv->abook_aggregator),
             _abook_aggregator_ready, model, NULL);
-
-    g_signal_connect (priv->abook_aggregator, "contacts-added",
-        G_CALLBACK (aggregator_contacts_changed_cb), model);
-    g_signal_connect (priv->abook_aggregator, "contacts-removed",
-        G_CALLBACK (aggregator_contacts_changed_cb), model);
 }
 
 static void
@@ -1332,7 +1340,7 @@ _new_event_callback (
     RTComLogModelPrivate * priv = NULL;
     RTComElQuery * query = NULL;
     RTComElIter * el_iter = NULL;
-    GValueArray * values = NULL;
+    GHashTable * values = NULL;
 
     g_return_if_fail(RTCOM_IS_LOG_MODEL(model));
 
@@ -1381,7 +1389,7 @@ _new_event_callback (
 
     if(el_iter && rtcom_el_iter_first(el_iter))
     {
-        values = rtcom_el_iter_get_valuearray(
+        values = rtcom_el_iter_get_value_map(
                 el_iter,
                 "id",
                 "service",
@@ -1396,6 +1404,8 @@ _new_event_callback (
                 "event-count",
                 "group-title",
                 "event-type",
+                "outgoing",
+                "flags",
                 NULL);
         g_object_unref(el_iter);
     }
@@ -1404,7 +1414,8 @@ _new_event_callback (
     {
         caching_data_t * d = NULL;
         const gchar * remote_ebook_uid;
-        remote_ebook_uid = g_value_get_string(g_value_array_get_nth(values, 6));
+        remote_ebook_uid = g_value_get_string(g_hash_table_lookup(values,
+              "remote-ebook-uid"));
         if(remote_ebook_uid)
         {
             g_debug("The new event has a remote_ebook_uid (%s): creating the "
@@ -1552,27 +1563,19 @@ _event_updated_callback (
     el_iter = rtcom_el_get_events(backend, query);
     if(el_iter && rtcom_el_iter_first(el_iter))
     {
-        GValueArray * values;
+        gchar *icon_name, *text, *remote_name, *group_title;
+        guint event_count;
 
-        if((values = rtcom_el_iter_get_valuearray(
-                el_iter,
-                "icon-name",
-                "content",
-                "remote-name",
-                "event-count",
-                "group-title",
-                NULL)))
+        if(rtcom_el_iter_get_values(el_iter,
+                "icon-name", &icon_name,
+                "content", &text,
+                "remote-name", &remote_name,
+                "event-count", &event_count,
+                "group-title", &group_title,
+                NULL))
         {
-            const gchar * icon_name, * text, * remote_name, *group_title;
-            guint event_count;
             GdkPixbuf * icon = NULL;
 
-            icon_name = g_value_get_string(g_value_array_get_nth(values, 0));
-            text = g_value_get_string(g_value_array_get_nth(values, 1));
-            remote_name = g_value_get_string(g_value_array_get_nth(values, 2));
-            event_count = g_value_get_int(g_value_array_get_nth(values, 3));
-            group_title = g_value_get_string(g_value_array_get_nth(values, 4));
-
             g_debug("Got icon_name=\"%s\", text=\"%s\", "
                     "remote_name = \"%s\", event_count = %d, "
                     "group_title = \"%s\"",
@@ -1610,7 +1613,10 @@ _event_updated_callback (
                     RTCOM_LOG_VIEW_COL_GROUP_TITLE, group_title,
                     -1);
 
-            g_value_array_free (values);
+            g_free (icon_name);
+            g_free (text);
+            g_free (remote_name);
+            g_free (group_title);
         }
 
         g_object_unref(el_iter);
@@ -1711,7 +1717,7 @@ _event_deleted_callback (
 
                 RTComElQuery   * query;
                 RTComElIter    * el_iter;
-                GValueArray    * values;
+                GHashTable    * values;
 
                 gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 
@@ -1734,7 +1740,7 @@ _event_deleted_callback (
                     caching_data_t * d;
                     RTComLogModelPrivate * priv;
 
-                    values = rtcom_el_iter_get_valuearray(
+                    values = rtcom_el_iter_get_value_map(
                             el_iter,
                             "id",
                             "service",
@@ -1749,6 +1755,8 @@ _event_deleted_callback (
                             "event-count",
                             "group-title",
                             "event-type",
+                            "outgoing",
+                            "flags",
                             NULL);
 
                     d = g_new0(caching_data_t, 1);
@@ -1827,7 +1835,7 @@ _event_deleted_callback (
 
                 RTComElQuery   * query;
                 RTComElIter    * el_iter;
-                GValueArray    * values;
+                GHashTable    * values;
 
                 gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 
@@ -1849,7 +1857,7 @@ _event_deleted_callback (
                     caching_data_t * d;
                     RTComLogModelPrivate * priv;
 
-                    values = rtcom_el_iter_get_valuearray(
+                    values = rtcom_el_iter_get_value_map(
                             el_iter,
                             "id",
                             "service",
@@ -1864,6 +1872,8 @@ _event_deleted_callback (
                             "event-count",
                             "group-title",
                             "event-type",
+                            "outgoing",
+                            "flags",
                             NULL);
 
                     d = g_new0(caching_data_t, 1);
@@ -1913,9 +1923,8 @@ _event_deleted_callback (
 
             RTComElQuery * query;
             RTComElIter  * el_iter;
-            GValueArray  * values;
             gint           new_id = -1;
-            const gchar  * icon_name = NULL,
+            gchar        * icon_name = NULL,
                          * text = NULL,
                          * remote_name = NULL;
             GdkPixbuf    * icon = NULL;
@@ -1987,6 +1996,10 @@ _event_deleted_callback (
              */
             if(el_iter && rtcom_el_iter_first(el_iter))
             {
+                gboolean outgoing;
+                const gchar *event_type;
+                gint flags;
+
                 /**
                  * Now notice that we're getting the id from the iterator.
                  * Since queries are also performed sorting by descending
@@ -1994,23 +2007,20 @@ _event_deleted_callback (
                  * group, hence the id of the event we're setting as
                  * group representative.
                  */
-                if((values = rtcom_el_iter_get_valuearray(
+
+                if(rtcom_el_iter_get_values(
                         el_iter,
-                        "id",
-                        "icon-name",
-                        "content",
-                        "remote-name",
-                        "event-type",
-                        NULL)))
+                        "id", &new_id,
+                        "icon-name", &icon_name,
+                        "content", &text,
+                        "remote-name", &remote_name,
+                        "event-type", &event_type,
+                        "outgoing", &outgoing,
+                        "flags", &flags,
+                        NULL))
                 {
                     GtkTreeIter resort_iter;
-                    const gchar *event_type;
 
-                    new_id = g_value_get_int(g_value_array_get_nth(values, 0));
-                    icon_name = g_value_get_string(g_value_array_get_nth(values, 1));
-                    text = g_value_get_string(g_value_array_get_nth(values, 2));
-                    remote_name = g_value_get_string(g_value_array_get_nth(values, 3));
-                    event_type = g_value_get_string(g_value_array_get_nth(values, 4));
                     g_debug("Got id=%d, icon_name=\"%s\", text=\"%s\", "
                             "remote_name = \"%s\" and event_type = \"%s\".",
                             new_id, icon_name, text, remote_name, event_type);
@@ -2042,6 +2052,8 @@ _event_deleted_callback (
                             RTCOM_LOG_VIEW_COL_TEXT, text,
                             RTCOM_LOG_VIEW_COL_REMOTE_NAME, remote_name,
                             RTCOM_LOG_VIEW_COL_EVENT_TYPE, event_type,
+                            RTCOM_LOG_VIEW_COL_OUTGOING, outgoing,
+                            RTCOM_LOG_VIEW_COL_FLAGS, flags,
                             -1);
 
                     /**
@@ -2073,7 +2085,9 @@ _event_deleted_callback (
                             break;
                     }
 
-                    g_value_array_free (values);
+                    g_free (icon_name);
+                    g_free (text);
+                    g_free (remote_name);
                 }
 
            }
@@ -2139,6 +2153,10 @@ refresh_idle_cb (gpointer model)
 {
     RTComLogModelPrivate * priv = RTCOM_LOG_MODEL_GET_PRIV(model);
 
+    /* If refresh-hint was a result of modifying contacts, our
+     * cache is stale. */
+    g_hash_table_remove_all (priv->cached_account_data);
+
     rtcom_log_model_refresh(model);
     priv->refresh_id = 0;
 
@@ -2306,7 +2324,9 @@ rtcom_log_model_init(
         RTCOM_LOG_VIEW_COL_TYPE_12,
         RTCOM_LOG_VIEW_COL_TYPE_13,
         RTCOM_LOG_VIEW_COL_TYPE_14,
-        RTCOM_LOG_VIEW_COL_TYPE_15
+        RTCOM_LOG_VIEW_COL_TYPE_15,
+        RTCOM_LOG_VIEW_COL_TYPE_16,
+        RTCOM_LOG_VIEW_COL_TYPE_17
     };
 
     gtk_list_store_set_column_types(
@@ -2338,6 +2358,7 @@ rtcom_log_model_init(
 
     priv->group_by = RTCOM_EL_QUERY_GROUP_BY_NONE;
     priv->limit = -1;
+    priv->show_group_chat = FALSE;
 
     priv->aggregator_ready_closure = NULL;
     priv->abook_aggregator = NULL;
@@ -2618,7 +2639,7 @@ rtcom_log_model_populate_query(
 {
     RTComLogModelPrivate * priv = NULL;
     RTComElIter * it = NULL;
-    GValueArray * values = NULL;
+    GHashTable * values = NULL;
     caching_data_t * d = NULL;
     guint event_count = 0;
     gint limit;
@@ -2664,7 +2685,7 @@ rtcom_log_model_populate_query(
         do
         {
             event_count++;
-            values = rtcom_el_iter_get_valuearray(
+            values = rtcom_el_iter_get_value_map(
                     it,
                     "id",
                     "service",
@@ -2679,6 +2700,8 @@ rtcom_log_model_populate_query(
                     "event-count",
                     "group-title",
                     "event-type",
+                    "outgoing",
+                    "flags",
                     NULL);
             d->cached_events = g_list_prepend(d->cached_events, values);
         } while(rtcom_el_iter_next(it));
@@ -2965,6 +2988,134 @@ _create_abook_account_manager (RTComLogModel *model)
     }
 }
 
+static gchar *utf8_next_token (gchar *txt)
+{
+    if (txt == NULL)
+        return NULL;
+
+    /* find first space character */
+    while ((*txt != '\0') && !g_unichar_isspace (g_utf8_get_char (txt)))
+        txt = g_utf8_find_next_char (txt, NULL);
+
+    /* skip (possibly several) space character(s) */
+    while (g_unichar_isspace (g_utf8_get_char (txt)))
+        txt = g_utf8_find_next_char (txt, NULL);
+
+    if (*txt == '\0')
+        return NULL;
+
+    return txt;
+}
+
+static gssize get_needle_token_size (gchar *needle)
+{
+    gssize count = 0;
+
+    while ((*needle != '\0') && !g_unichar_isspace (g_utf8_get_char (needle)))
+    {
+        count++;
+        needle = g_utf8_find_next_char (needle, NULL);
+    }
+
+    return count;
+}
+
+static gboolean utf8_startswith (const gchar *haystack,
+    const gchar *needle)
+{
+    while ((*haystack != '\0') && (*needle != '\0'))
+    {
+        if (g_utf8_get_char (haystack) != g_utf8_get_char (needle))
+            return FALSE;
+
+        haystack = g_utf8_next_char (haystack);
+        needle = g_utf8_next_char (needle);
+    }
+
+    if (*needle != '\0')
+        return FALSE;
+
+    return TRUE;
+}
+
+
+gboolean
+rtcom_log_model_filter_visible_func (GtkTreeModel *model,
+    GtkTreeIter *iter, gchar *text, gpointer data)
+{
+    glong len = 0;
+    gssize size;
+    gchar * displayed_name;
+    gboolean result = FALSE;
+    gchar * haystack;
+    gchar * haypart;
+    gchar * needle;
+    gchar * needle_part;
+    gchar * needle_token;
+
+    g_debug ("%s: called for %s", G_STRFUNC, text);
+
+    len = g_utf8_strlen(text, -1);
+
+    if(len == 0)
+        return TRUE;
+
+    gtk_tree_model_get(model, iter,
+            RTCOM_LOG_VIEW_COL_REMOTE_NAME, &displayed_name,
+            -1);
+
+    if (!displayed_name)
+        gtk_tree_model_get (model, iter,
+            RTCOM_LOG_VIEW_COL_REMOTE_ACCOUNT, &displayed_name, -1);
+
+    needle = g_utf8_strdown(text, -1);
+    haystack = g_utf8_strdown(displayed_name, -1);
+    g_free (displayed_name);
+
+    /*tokenizing needle*/
+    for (needle_part = needle;
+        needle_part != NULL;
+        needle_part = utf8_next_token (needle_part))
+    {
+        result = FALSE;
+        size = get_needle_token_size (needle_part);
+        needle_token = g_utf8_strdown (needle_part, size);
+
+        for (haypart = haystack;
+            haypart != NULL;
+            haypart = utf8_next_token (haypart))
+        {
+            if (utf8_startswith (haypart, needle_token))
+            {
+                result = TRUE;
+                break;
+            }
+        }
+        g_free (needle_token);
+        if (result == FALSE)
+            break;
+    }
+
+    g_free (haystack);
+    g_free (needle);
+
+    return result;
+}
+
+void
+rtcom_log_model_set_show_group_chat (
+        RTComLogModel * model,
+        gboolean is_shown)
+{
+    RTComLogModelPrivate *priv;
+
+    g_return_if_fail (RTCOM_IS_LOG_MODEL (model));
+    priv = RTCOM_LOG_MODEL_GET_PRIV (model);
+
+    priv->show_group_chat = is_shown;
+    rtcom_log_model_refresh(model);
+}
+
 
 /* vim: set ai et tw=75 ts=4 sw=4: */
 
