@@ -35,6 +35,8 @@
 #include <clockd/libtime.h>
 #include <dbus/dbus.h>
 
+#include <rtcom-eventlogger-plugins/chat.h>
+
 static void
 rtcom_log_view_class_init(
         RTComLogViewClass * klass);
@@ -97,10 +99,6 @@ struct _RTComLogViewPrivate
     guint disposed : 1;
     gboolean highlight_new_events;
 
-    /* A flag to control whether to show group chat
-     * specific information (avatar, group_title)*/
-    gboolean show_group_chat;
-
     /* A flag to control whether to show display names
      * (default) or uids/phone numbers. */
     gboolean show_display_names;
@@ -211,6 +209,7 @@ static DBusHandlerResult _dbus_tz_change_cb (
 
     g_debug ("%s: called, clearing text cell cache", G_STRFUNC);
 
+    tzset ();
     g_hash_table_remove_all (priv->text_cell_cache);
     gtk_widget_queue_draw (GTK_WIDGET (view));
 
@@ -338,7 +337,6 @@ rtcom_log_view_init(
     priv->current_width                = 0;
     priv->fixed_width                  = -1;
     priv->highlight_new_events         = TRUE;
-    priv->show_group_chat              = FALSE;
     priv->show_display_names           = TRUE;
 
     priv->text_cell_cache = g_hash_table_new_full
@@ -697,13 +695,34 @@ rtcom_log_view_set_model(
 
 /* Some private functions */
 static gboolean
-is_group_chat (
-        RTComLogViewPrivate * priv,
-        const gchar *group_uid)
+is_group_chat (GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
-    return (priv->show_group_chat &&
-            group_uid && group_uid[0] &&
-            g_str_has_prefix (group_uid, "group:"));
+    gint flags;
+    gboolean result;
+
+    gtk_tree_model_get (tree_model, iter,
+        RTCOM_LOG_VIEW_COL_FLAGS, &flags, -1);
+
+    if (flags & RTCOM_EL_FLAG_CHAT_GROUP)
+      {
+        gchar *service;
+
+        gtk_tree_model_get (tree_model, iter,
+            RTCOM_LOG_VIEW_COL_SERVICE, &service, -1);
+
+        if (strcmp (service, "RTCOM_EL_SERVICE_CHAT"))
+            result = FALSE;
+        else
+            result = TRUE;
+
+        g_free (service);
+      }
+    else
+      {
+        result = FALSE;
+      }
+
+    return result;
 }
 
 static void
@@ -740,36 +759,37 @@ _presence_cell_func(
     OssoABookPresence * presence = NULL;
     const gchar * presence_icon = NULL;
     RTComLogViewPrivate * priv;
-    gchar * group_uid;
 
     priv = RTCOM_LOG_VIEW_GET_PRIV(data);
 
+    if (is_group_chat (tree_model, iter))
+    {
+        g_object_set(cell, "pixbuf", pixbuf, NULL);
+        return;
+    }
+
     gtk_tree_model_get (tree_model, iter,
                         RTCOM_LOG_VIEW_COL_CONTACT, &presence,
-                        RTCOM_LOG_VIEW_COL_GROUP_UID, &group_uid,
                         -1);
 
-    if (!is_group_chat (priv, group_uid))
+    if(presence)
+        presence_icon = osso_abook_presence_get_icon_name(presence);
+
+    if (presence_icon)
     {
-        if(presence)
-            presence_icon = osso_abook_presence_get_icon_name(presence);
-
-        if (presence_icon)
+        pixbuf = g_hash_table_lookup (priv->presence_icon_cache,
+                presence_icon);
+        if (!pixbuf)
         {
-            pixbuf = g_hash_table_lookup (priv->presence_icon_cache,
-                    presence_icon);
-            if (!pixbuf)
-            {
-                pixbuf = gtk_icon_theme_load_icon
-                    (gtk_icon_theme_get_default (),
-                     presence_icon,
-                     HILDON_ICON_PIXEL_SIZE_XSMALL,
-                     0, NULL);
+            pixbuf = gtk_icon_theme_load_icon
+                (gtk_icon_theme_get_default (),
+                 presence_icon,
+                 HILDON_ICON_PIXEL_SIZE_XSMALL,
+                 0, NULL);
 
-                if (pixbuf)
-                    g_hash_table_insert (priv->presence_icon_cache,
-                            g_strdup (presence_icon), pixbuf);
-            }
+            if (pixbuf)
+                g_hash_table_insert (priv->presence_icon_cache,
+                        g_strdup (presence_icon), pixbuf);
         }
     }
 
@@ -780,7 +800,6 @@ _presence_cell_func(
 
     if (presence)
         g_object_unref (presence);
-    g_free (group_uid);
 }
 
 static void
@@ -904,7 +923,6 @@ _text_cell_func(
   gchar *text;
   gint timestamp;
   gint count;
-  gchar *group_uid;
   gchar *group_title;
   gchar *markup;
   gchar *count_str = NULL;
@@ -939,17 +957,13 @@ _text_cell_func(
           RTCOM_LOG_VIEW_COL_REMOTE_ACCOUNT, &remote_uid,
           RTCOM_LOG_VIEW_COL_TIMESTAMP, &timestamp,
           RTCOM_LOG_VIEW_COL_COUNT, &count,
-          RTCOM_LOG_VIEW_COL_GROUP_UID, &group_uid,
           RTCOM_LOG_VIEW_COL_GROUP_TITLE, &group_title,
           -1);
 
   if (count > 1)
       count_str = g_strdup_printf (" (%d)", count);
 
-  /* FIXME: we don't need group_uid later when we use
-   * EVENT_TYPE to identify group chat */
-  if (group_title && (*group_title != '\0') &&
-      is_group_chat (priv, group_uid))
+  if (is_group_chat (tree_model, iter))
       name_str = group_title;
 
   if (priv->show_display_names &&
@@ -1002,7 +1016,6 @@ _text_cell_func(
   g_free (text);
   g_free (remote_name);
   g_free (remote_uid);
-  g_free (group_uid);
   g_free (group_title);
   g_free (count_str);
 
@@ -1024,17 +1037,14 @@ _avatar_cell_func(
 {
     OssoABookAvatar * avatar = NULL;
     GdkPixbuf * avatar_pixbuf = NULL;
-    gchar *group_uid = NULL;
-    RTComLogViewPrivate * priv = RTCOM_LOG_VIEW_GET_PRIV (data);
 
     gtk_tree_model_get(
             tree_model,
             iter,
             RTCOM_LOG_VIEW_COL_CONTACT, &avatar,
-            RTCOM_LOG_VIEW_COL_GROUP_UID, &group_uid,
             -1);
 
-    if (is_group_chat (priv, group_uid))
+    if(is_group_chat(tree_model, iter))
     {
         static GdkPixbuf *muc_pixbuf = NULL;
 
@@ -1071,8 +1081,6 @@ _avatar_cell_func(
     }
 
     g_object_set(cell, "pixbuf", avatar_pixbuf, NULL);
-
-    g_free (group_uid);
 }
 
 static void
@@ -1181,8 +1189,12 @@ rtcom_log_view_set_show_group_chat (
     g_return_if_fail (RTCOM_IS_LOG_VIEW (view));
     priv = RTCOM_LOG_VIEW_GET_PRIV (view);
 
-    priv->show_group_chat = is_shown;
-    g_hash_table_remove_all (priv->text_cell_cache);
+    if (priv->model != NULL)
+    {
+        rtcom_log_model_set_show_group_chat (RTCOM_LOG_MODEL (priv->model),
+            is_shown);
+        g_hash_table_remove_all (priv->text_cell_cache);
+    }
 }
 
 void
